package nl.kallestruik.riseofblocks.blocks;

import net.minecraft.core.BlockPos;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.DyeColor;
import net.minecraft.world.item.context.BlockPlaceContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.EnumProperty;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import nl.kallestruik.riseofblocks.blockentities.FlagBlockEntity;
import nl.kallestruik.riseofblocks.registry.FlagRegistry;
import nl.kallestruik.riseofblocks.util.TeamUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.annotation.ParametersAreNonnullByDefault;

public class FlagBlock extends Block implements EntityBlock {
    public static final EnumProperty<DyeColor> COLOR = EnumProperty.create("color", DyeColor.class);
    private static final VoxelShape SHAPE = Shapes.or(
            Block.box(2, 0, 2, 14, 1, 14),
            Block.box(7, 1, 7, 9, 24, 9),
            Block.box(8, 17, 7, 16, 23, 9)
    );

    public FlagBlock(Properties properties) {
        super(properties.noCollission().noOcclusion().explosionResistance(1000).noLootTable());
        this.registerDefaultState(this.stateDefinition.any().setValue(COLOR, DyeColor.WHITE));
    }

    @Override
    protected void createBlockStateDefinition(StateDefinition.Builder<Block, BlockState> builder) {
        builder.add(COLOR);
    }

    @Override
    public @Nullable BlockState getStateForPlacement(BlockPlaceContext context) {
        Player player = context.getPlayer();
        if (player == null) {
            return this.defaultBlockState();
        }
        DyeColor color = TeamUtil.getDyeColorForTeam(player.getTeam());
        return this.stateDefinition.any().setValue(COLOR, color);
    }

    @Override
    protected VoxelShape getShape(BlockState state, BlockGetter level, BlockPos pos, CollisionContext context) {
        return SHAPE;
    }

    @Override
    @ParametersAreNonnullByDefault
    public BlockEntity newBlockEntity(BlockPos pos, BlockState state) {
        return new FlagBlockEntity(pos, state);
    }

    @Override
    @ParametersAreNonnullByDefault
    public @NotNull BlockState playerWillDestroy(Level level, BlockPos pos, BlockState state, Player player) {
        if (!level.isClientSide && level instanceof ServerLevel serverLevel) {
            BlockEntity be = serverLevel.getBlockEntity(pos);
            if (be instanceof FlagBlockEntity blockEntity) {
                String teamName = blockEntity.getTeamName();
                if (teamName != null) {
                    FlagRegistry.get(serverLevel.getServer()).remove(teamName);
                }
            }
        }
        return super.playerWillDestroy(level, pos, state, player);
    }
}
