package nl.kallestruik.riseofblocks.commands;

import com.mojang.brigadier.CommandDispatcher;
import com.mojang.brigadier.context.CommandContext;
import net.minecraft.ChatFormatting;
import net.minecraft.commands.CommandBuildContext;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.Commands;
import net.minecraft.core.BlockPos;
import net.minecraft.network.chat.ClickEvent;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.HoverEvent;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.MinecraftServer;
import net.minecraft.world.scores.Team;
import nl.kallestruik.riseofblocks.registry.FlagRegistry;

import java.util.Map;

public class FlagCommand {

    public static void registerCommands(CommandDispatcher<CommandSourceStack> dispatcher, CommandBuildContext context, Commands.CommandSelection selection) {
        dispatcher.register(
                Commands.literal("flags")
                        .executes(FlagCommand::listFlags)
                        .then(Commands.literal("admin")
                                .requires(src -> src.hasPermission(2)) // OP only
                                .executes(FlagCommand::listFlagsAdmin))
        );
    }

    public static int listFlags(CommandContext<CommandSourceStack> context) {
        context.getSource().sendSystemMessage(Component.literal("Not available during the grace period."));
        return 1;
    }

    public static int listFlagsAdmin(CommandContext<CommandSourceStack> context) {
        MinecraftServer server = context.getSource().getServer();
        for (Map.Entry<String, FlagRegistry.FlagPosition> flags : FlagRegistry.get(server).getAll()) {
            Team team = server.getScoreboard().getPlayerTeam(flags.getKey());
            if (team == null) continue;
            Component flagTeamComponent = Component.literal(team.getName()).withStyle(team.getColor());
            BlockPos pos = flags.getValue().getPos();

            String posStr = pos.getX() + " " + pos.getY() + " " + pos.getZ();

            MutableComponent coordComponent = Component.literal("[" + posStr + "]")
                    .withStyle(style -> style
                            .withColor(ChatFormatting.AQUA)
                            .withClickEvent(new ClickEvent(ClickEvent.Action.RUN_COMMAND, "/tp @s " + posStr))
                            .withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT,
                                    Component.literal("Click to teleport to " + posStr)))
                    );

            context.getSource().sendSystemMessage(Component.empty().append(flagTeamComponent).append(Component.literal(": ")).append(coordComponent));
        }
        return 1;
    }
}
